@@ -590,7 +590,89 @@ def centered_indexes_to_normal_indexes(
     return normal_indexes
 
 
-class Range:
+class IntRange:
+
+    def __init__(self, raw_range):
+
+        if isinstance(raw_range, IntRange):
+            start = raw_range.start
+            stop = raw_range.stop
+            step = raw_range.step
+
+        elif isinstance(raw_range, dict):
+            keys = ['start', 'stop', 'step']
+            for k in raw_range.keys():
+                if k in keys:
+                    continue
+                else:
+                    raise KeyError(k)
+            if raw_range.get('start') is None:
+                start = 0
+            else:
+                start = raw_range['start']
+
+            if raw_range.get('stop') is None:
+                raise ValueError("raw_range['stop']")
+            else:
+                stop = raw_range['stop']
+
+            if raw_range.get('step') is None:
+                step = 1
+            else:
+                step = raw_range['step']
+
+        elif isinstance(raw_range, range):
+            start = raw_range.start
+            stop = raw_range.stop
+            step = raw_range.step
+        elif isinstance(raw_range, (list, tuple, np.ndarray)):
+            n_elements = len(raw_range)
+            if n_elements == 1:
+                start = 0
+                stop = raw_range[0]
+                step = 1
+            elif n_elements == 2:
+                start, stop = raw_range
+                step = 1
+            elif n_elements == 3:
+                start, stop, step = raw_range
+            else:
+                raise ValueError('raw_range has to have at least 1 element. {} are given'.format(n_elements))
+        elif isinstance(raw_range, numbers.Number):
+            start = 0
+            stop = raw_range
+            step = 1
+        else:
+            raise TypeError('raw_range')
+
+        self.start = start
+        self.stop = stop
+        self.step = step
+        if self.stop > self.start:
+            if self.step > 0:
+                self.len = math.ceil((self.stop - self.start) / self.step)
+            elif self.step < 0:
+                self.len = 0
+            else:
+                raise ValueError('self.step')
+        elif self.stop < self.start:
+            if self.step < 0:
+                self.len = math.ceil(abs((self.stop - self.start) / self.step))
+            elif self.step > 0:
+                self.len = 0
+            else:
+                raise ValueError('self.step')
+        else:
+            self.len = 0
+
+    def to_slice(self):
+        return slice(self.start, self.stop, self.step)
+
+    def to_list(self):
+        return list(range(self.start, self.stop, self.step))
+
+
+class FloatRange:
     def __init__(self, raw_range):
         if isinstance(raw_range, dict):
             keys = ['start', 'stop', 'step']
@@ -631,7 +713,7 @@ class Range:
                 start, stop, step = raw_range
             else:
                 raise ValueError('raw_range has to have at least 1 element. {} are given'.format(n_elements))
-        elif isinstance(raw_range, (numbers.Number, None)):
+        elif isinstance(raw_range, numbers.Number) or raw_range is None:
             start = 0
             stop = raw_range
             step = 1
@@ -641,7 +723,7 @@ class Range:
         self.start = cc_maths.convert_to_int_or_float(start)
         self.stop = cc_maths.convert_to_int_or_float(stop)
         self.step = cc_maths.convert_to_int_or_float(step)
-        self.len = cc_maths.convert_to_int_or_float((self.stop - self.start) / self.step)
+        self.len = abs(cc_maths.convert_to_int_or_float((self.stop - self.start) / self.step))
 
     # def __init__(self, stop, *args):
     #
@@ -693,5 +775,3 @@ class Range:
             step=len_each_equal_ranges)
         return centers
 
-    def to_slice(self):
-        return slice(self.start, self.stop, self.step)
