@@ -12,7 +12,7 @@ def n_conditions_to_combinations(
         n_repetitions_combinations=1,
         order_variables='rl',
         variables_in_order=None,
-        dtype=int):
+        dtype='i'):
 
     # order_variables is the order of the variables by which they change their own conditions.
     # order_variables can either be 'rl' for right-to-left, 'lr' for left-to-right or 'c' for custom.
@@ -31,14 +31,14 @@ def n_conditions_to_combinations(
         variables = np.arange(n_variables)
         # n_conditions = np.asarray(n_conditions, dtype=int)
         # conditions = n_conditions_to_conditions(n_conditions)
-        n_combinations = np.prod(n_conditions) * n_repetitions_combinations
+        n_combinations = cc_maths.prod(n_conditions) * n_repetitions_combinations
         if n_combinations < 0:
             n_conditions = n_conditions.astype(np.int64)
-            n_combinations = np.prod(n_conditions) * n_repetitions_combinations
+            n_combinations = cc_maths.prod(n_conditions) * n_repetitions_combinations
 
         # axis_combinations = int(not(bool(axis_variables)))
         axis_variables = int(not(bool(axis_combinations)))
-        shape_combinations = np.empty(2, dtype=int)
+        shape_combinations = np.empty(2, dtype='i')
         shape_combinations[axis_combinations] = n_combinations
         shape_combinations[axis_variables] = n_variables
 
@@ -49,7 +49,7 @@ def n_conditions_to_combinations(
         elif order_variables == 'lr':
             variables_in_order = variables
         elif order_variables == 'c':
-            variables_in_order = np.asarray(variables_in_order, dtype=int)
+            variables_in_order = np.asarray(variables_in_order, dtype='i')
             # check requirement 2
             if len(variables_in_order) != n_variables:
                 raise ValueError('The following condition must be met:\nlen(variables_in_order) == len(n_conditions).')
@@ -109,16 +109,17 @@ def n_conditions_to_combinations(
 
 
 def n_conditions_to_combinations_on_the_fly(
-        n_conditions, order_variables='rl', variables_in_order=None, dtype=int):
+        n_conditions, order_variables='rl', variables_in_order=None, dtype='i'):
     try:
         n_variables = len(n_conditions)
         if not isinstance(n_conditions, np.ndarray):
-            n_conditions = np.asarray(n_conditions, dtype=int)
+            n_conditions = np.asarray(n_conditions, dtype='i')
     except TypeError:
-        n_conditions = np.asarray([n_conditions], dtype=int)
+        n_conditions = np.asarray([n_conditions], dtype='i')
         n_variables = 1
 
-    n_combinations = int(np.prod(n_conditions))
+    # n_combinations = int(np.prod(n_conditions))
+    n_combinations = cc_maths.prod(n_conditions)
 
     combination_i = np.empty(n_variables, dtype=dtype)
     combination_i[:] = 0
@@ -141,59 +142,69 @@ def n_conditions_to_combinations_on_the_fly(
         yield combination_i
 
 
-def conditions_to_combinations_on_the_fly_with_indexes(
-        conditions, order_variables='rl', variables_in_order=None, dtype='i'):
-
-
-    n_variables = len(conditions)
-    if not isinstance(conditions, np.ndarray):
-        conditions = np.asarray(conditions, dtype=object)
-
-    n_conditions = conditions_to_n_conditions(conditions)
-    n_combinations = int(np.prod(n_conditions))
-
-    indexes_combination_i = np.empty(n_variables, dtype=dtype)
-    indexes_combination_i[:] = 0
-    combination_i = np.empty(n_variables, dtype=dtype)
-    for v in range(n_variables):
-        combination_i[v] = conditions[v][indexes_combination_i[v]]
-    yield indexes_combination_i, combination_i
-
-    for i in range(1, n_combinations):
-
-        v = n_variables - 1
-        increase_condition_v = True
-
-        while increase_condition_v:
-            indexes_combination_i[v] += 1
-            if indexes_combination_i[v] >= n_conditions[v]:
-                indexes_combination_i[v] = 0
-                v -= 1
-            else:
-                increase_condition_v = False
-
-        for v in range(n_variables):
-            combination_i[v] = conditions[v][indexes_combination_i[v]]
-
-        yield indexes_combination_i, combination_i
-
-
 def conditions_to_combinations_on_the_fly(
-        conditions, order_variables='rl', variables_in_order=None, dtype='i'):
+        conditions_raw, order_variables='rl', variables_in_order=None, dtype=None, order_outputs='vi'):
+    """
+    Parameters
+    ----------
+    order_outputs : str or sequence of str, optional
+        The desired outputs. Accepted values are "v", "i" or any combination of them like "vi", "iv" (default is "vi").
+        "v" stands for combination_values_i and "i" for combination_indexes_i.
+    """
 
-    n_variables = len(conditions)
-    if not isinstance(conditions, np.ndarray):
-        conditions = np.asarray(conditions, dtype=object)
+    if not isinstance(conditions_raw, np.ndarray):
+        conditions = np.asarray(conditions_raw, dtype=object)
+    else:
+        conditions = conditions_raw
 
     n_conditions = conditions_to_n_conditions(conditions)
-    n_combinations = int(np.prod(n_conditions))
+    n_combinations = cc_maths.prod(n_conditions)
+    n_variables = len(n_conditions)
 
-    indexes_combination_i = np.empty(n_variables, dtype=dtype)
-    indexes_combination_i[:] = 0
-    combination_i = np.empty(n_variables, dtype=dtype)
-    for v in range(n_variables):
-        combination_i[v] = conditions[v][indexes_combination_i[v]]
-    yield combination_i
+    combination_indexes_i = np.empty(n_variables, dtype='i')
+    combination_indexes_i[:] = 0
+
+    order_accepted_values = 'vi'
+    if order_outputs is None:
+        order_outputs = 'vi'
+        n_outputs = 2
+    else:
+        n_outputs = len(order_outputs)
+        if n_outputs < 1:
+            raise ValueError('order_outputs')
+        else:
+            for o in range(n_outputs):
+                if not (order_outputs[o] in order_accepted_values):
+                    raise ValueError('order_outputs')
+
+    outputs_i = [None] * n_outputs  # type: list
+
+    for o in range(n_outputs):
+
+        if order_outputs[o] == 'v':
+
+            dtype_np = np.dtype(dtype)
+            if dtype_np.kind == 'U':
+                combination_values_i = np.empty(n_variables, dtype='O')
+            else:
+                combination_values_i = np.empty(n_variables, dtype=dtype)
+
+            for v in range(n_variables):
+                combination_values_i[v] = conditions[v][combination_indexes_i[v]]
+
+            if dtype_np.kind == 'U':
+                outputs_i[o] = combination_values_i.astype('U')
+            else:
+                outputs_i[o] = combination_values_i
+
+        elif order_outputs[o] == 'i':
+
+            outputs_i[o] = combination_indexes_i
+
+    if n_outputs == 1:
+        yield outputs_i[0]
+    else:
+        yield outputs_i
 
     for i in range(1, n_combinations):
 
@@ -201,17 +212,43 @@ def conditions_to_combinations_on_the_fly(
         increase_condition_v = True
 
         while increase_condition_v:
-            indexes_combination_i[v] += 1
-            if indexes_combination_i[v] >= n_conditions[v]:
-                indexes_combination_i[v] = 0
+            combination_indexes_i[v] += 1
+            if combination_indexes_i[v] >= n_conditions[v]:
+                combination_indexes_i[v] = 0
                 v -= 1
             else:
                 increase_condition_v = False
 
-        for v in range(n_variables):
-            combination_i[v] = conditions[v][indexes_combination_i[v]]
+        for o in range(n_outputs):
 
-        yield combination_i
+            if order_outputs[o] == 'v':
+
+                for v in range(n_variables):
+                    combination_values_i[v] = conditions[v][combination_indexes_i[v]]
+
+                if dtype_np.kind == 'U':
+                    outputs_i[o] = combination_values_i.astype('U')
+                else:
+                    outputs_i[o] = combination_values_i
+
+            elif order_outputs[o] == 'i':
+
+                outputs_i[o] = combination_indexes_i
+
+        if n_outputs == 1:
+            yield outputs_i[0]
+        else:
+            yield outputs_i
+
+
+# def conditions_to_combinations_on_the_fly(
+#         conditions, order_variables='rl', variables_in_order=None, dtype=None):
+#
+#     for _, combination_i in conditions_to_combinations_on_the_fly_with_indexes(
+#             conditions, order_variables=order_variables,
+#             variables_in_order=variables_in_order, dtype=dtype):
+#
+#         yield combination_i
 
 
 def conditions_to_combinations(
@@ -239,10 +276,10 @@ def conditions_to_combinations(
         variables = np.arange(n_variables)
         # conditions = np.asarray(conditions, dtype=object)
         n_conditions = conditions_to_n_conditions(conditions)
-        n_combinations = np.prod(n_conditions) * n_repetitions_combinations
+        n_combinations = cc_maths.prod(n_conditions) * n_repetitions_combinations
         if n_combinations < 0:
             n_conditions = n_conditions.astype(np.int64)
-            n_combinations = np.prod(n_conditions) * n_repetitions_combinations
+            n_combinations = cc_maths.prod(n_conditions) * n_repetitions_combinations
 
         if dtype is None:
             dtype = type(conditions[0][0])
@@ -562,9 +599,9 @@ def conditions_to_blocks_of_shuffled_combinations(
 
         conditions_variables_split = conditions[variables_split]
         n_conditions_variables_split = n_conditions[variables_split]
-        n_combinations_variables_split = np.prod(n_conditions_variables_split)
+        n_combinations_variables_split = cc_maths.prod(n_conditions_variables_split)
 
-    n_combinations_variables_repeated = int(np.prod(n_conditions_variables_repeated))
+    n_combinations_variables_repeated = cc_maths.prod(n_conditions_variables_repeated)
 
     n_trials_variables_repeated = n_combinations_variables_repeated * n_repetitions_combinations
 
@@ -632,7 +669,7 @@ def conditions_to_blocks_of_shuffled_combinations(
                 np.arange(n_variables_split) != i_variable_split]
 
             min_trials_of_1_single_variable_split_per_block[i_variable_split] = \
-                (n_combinations_variables_repeated * np.prod(
+                (n_combinations_variables_repeated * cc_maths.prod(
                     n_conditions_variables_split_not_in_variables_split) *
                  n_repetitions_combinations) // n_blocks
 
