@@ -10,7 +10,7 @@ my_dpi = 100
 
 
 def points_multi_formats_1_plot(
-        x, y, axes,
+        x_raw, y_raw, axes,
         marker='o', linestyle='-', color='b', linewidth=2, markersize=2,
         ax=None, n_pixels_x=300, n_pixels_y=300, legend=False, label_legend='', tight_layout=True, **kwargs):
 
@@ -40,104 +40,15 @@ def points_multi_formats_1_plot(
     if kwargs.get('label_y') is not None:
         ax.set_ylabel(kwargs['label_y'], fontsize=kwargs.get('fontsize_label_y'), rotation=kwargs.get('rotation_label_y'))
 
-    ticks_x_are_applied = False
-    if kwargs.get('labels_ticks_x') is None:
-        # max_x = data.shape[dict_axes['x']] - 1
-        if kwargs.get('ticks_x') is None:
-            if kwargs.get('n_ticks_x') is None:
-                pass
-            else:
-                max_x = x.max
-                min_x = x.min
-                n_labels_ticks_x = kwargs['n_ticks_x']
-                delta_x = (max_x - min_x) / (n_labels_ticks_x - 1)
-                tick_x_lower = min_x
-                tick_x_higher = max_x
-                kwargs['ticks_x'] = np.arange(tick_x_lower, tick_x_higher + delta_x, delta_x)
-                ax.set_xticks(kwargs['ticks_x'])
-                ticks_x_are_applied = True
-        else:
-            ax.set_xticks(kwargs['ticks_x'])
-            ticks_x_are_applied = True
+    if isinstance(x_raw, (list, tuple, range)):
+        x = np.asarray(x_raw)
+    else:
+        x = x_raw
 
-        if (kwargs.get('stagger_labels_ticks_x') or
-                (kwargs.get('fontsize_labels_ticks_x') is not None) or
-                (kwargs.get('rotation_labels_ticks_x') is not None)):
-            fig.canvas.draw()
-            tmp_labels_ticks_x = ax.get_xticklabels()
-            n_labels_ticks_x = len(tmp_labels_ticks_x)
-            kwargs['labels_ticks_x'] = [None] * n_labels_ticks_x
-            for l, label_l in enumerate(tmp_labels_ticks_x):
-                kwargs['labels_ticks_x'][l] = label_l.get_text()
-
-    if kwargs.get('labels_ticks_x') is not None:
-        if not ticks_x_are_applied:
-            if kwargs.get('ticks_x') is None:
-                max_x = x.max
-                min_x = x.min
-                n_labels_ticks_x = len(kwargs['labels_ticks_x'])
-                delta_x = (max_x - min_x) / (n_labels_ticks_x - 1)
-                tick_x_lower = min_x
-                tick_x_higher = max_x
-                kwargs['ticks_x'] = np.arange(tick_x_lower, tick_x_higher + delta_x, delta_x)
-
-            ax.set_xticks(kwargs['ticks_x'])
-        if kwargs.get('stagger_labels_ticks_x'):
-            kwargs['labels_ticks_x'] = (
-                [l if not i % 2 else '\n' + l for i, l in enumerate(kwargs['labels_ticks_x'])])
-        ax.set_xticklabels(
-            kwargs['labels_ticks_x'], fontsize=kwargs.get('fontsize_labels_ticks_x'),
-            rotation=kwargs.get('rotation_labels_ticks_x'))
-
-    ticks_y_are_applied = False
-    if kwargs.get('labels_ticks_y') is None:
-        # max_y = data.shape[dict_axes['y']] - 1
-        if kwargs.get('ticks_y') is None:
-            if kwargs.get('n_ticks_y') is None:
-                pass
-            else:
-                max_y = y.max
-                min_y = y.min
-                n_labels_ticks_y = kwargs['n_ticks_y']
-                delta_y = (max_y - min_y) / (n_labels_ticks_y - 1)
-                tick_y_lower = min_y
-                tick_y_higher = max_y
-                kwargs['ticks_y'] = np.arange(tick_y_lower, tick_y_higher + delta_y, delta_y)
-                ax.set_yticks(kwargs['ticks_y'])
-                ticks_y_are_applied = True
-
-        else:
-            ax.set_yticks(kwargs['ticks_y'])
-            ticks_y_are_applied = True
-
-        if (kwargs.get('stagger_labels_ticks_y') or
-                (kwargs.get('fontsize_labels_ticks_y') is not None) or
-                (kwargs.get('rotation_labels_ticks_y') is not None)):
-            fig.canvas.draw()
-            tmp_labels_ticks_y = ax.get_yticklabels()
-            n_labels_ticks_y = len(tmp_labels_ticks_y)
-            kwargs['labels_ticks_y'] = [None] * n_labels_ticks_y
-            for l, label_l in enumerate(tmp_labels_ticks_y):
-                kwargs['labels_ticks_y'][l] = label_l.get_text()
-
-    if kwargs.get('labels_ticks_y') is not None:
-        if not ticks_y_are_applied:
-            if kwargs.get('ticks_y') is None:
-                max_y = y.max
-                min_y = y.min
-                n_labels_ticks_y = len(kwargs['labels_ticks_y'])
-                delta_y = (max_y - min_y) / (n_labels_ticks_y - 1)
-                tick_y_lower = min_y
-                tick_y_higher = max_y
-                kwargs['ticks_y'] = np.arange(tick_y_lower, tick_y_higher + delta_y, delta_y)
-
-            ax.set_yticks(kwargs['ticks_y'])
-        if kwargs.get('stagger_labels_ticks_y'):
-            kwargs['labels_ticks_y'] = (
-                [l if not i % 2 else '\n' + l for i, l in enumerate(kwargs['labels_ticks_y'])])
-        ax.set_yticklabels(
-            kwargs['labels_ticks_y'], fontsize=kwargs.get('fontsize_labels_ticks_y'),
-            rotation=kwargs.get('rotation_labels_ticks_y'))
+    if isinstance(y_raw, (list, tuple, range)):
+        y = np.asarray(y_raw)
+    else:
+        y = y_raw
 
     shape_x = np.asarray(x.shape, dtype=int)
     n_dim_x = shape_x.size
@@ -177,11 +88,11 @@ def points_multi_formats_1_plot(
                 not isinstance(markersize, np.ndarray)):
             markersize = [markersize] * n_formats
 
-        if legend:
-            if (not isinstance(label_legend, list) and
-                    not isinstance(label_legend, tuple) and
-                    not isinstance(label_legend, np.ndarray)):
-                label_legend = [label_legend] * n_formats
+
+        if (not isinstance(label_legend, list) and
+                not isinstance(label_legend, tuple) and
+                not isinstance(label_legend, np.ndarray)):
+            label_legend = [label_legend] * n_formats
 
         indexes = np.empty(n_dim_y, dtype=object)
         indexes[axes['points']] = slice(0, None, 1)
@@ -237,11 +148,10 @@ def points_multi_formats_1_plot(
                     not isinstance(markersize, np.ndarray)):
                 markersize = [markersize] * n_formats
 
-            if legend:
-                if (not isinstance(label_legend, list) and
-                        not isinstance(label_legend, tuple) and
-                        not isinstance(label_legend, np.ndarray)):
-                    label_legend = [label_legend] * n_formats
+            if (not isinstance(label_legend, list) and
+                    not isinstance(label_legend, tuple) and
+                    not isinstance(label_legend, np.ndarray)):
+                label_legend = [label_legend] * n_formats
 
             indexes = np.empty(n_dim_x, dtype=object)
             indexes[axes['points']] = slice(0, None, 1)
@@ -279,11 +189,11 @@ def points_multi_formats_1_plot(
                     isinstance(markersize, np.ndarray)):
                 markersize = markersize[0]
 
-            if legend:
-                if (isinstance(label_legend, list) or
-                        isinstance(label_legend, tuple) or
-                        isinstance(label_legend, np.ndarray)):
-                    label_legend = label_legend[0]
+
+            if (isinstance(label_legend, list) or
+                    isinstance(label_legend, tuple) or
+                    isinstance(label_legend, np.ndarray)):
+                label_legend = label_legend[0]
 
             points_1_format_in_1_plot(
                 x, y,
@@ -296,6 +206,106 @@ def points_multi_formats_1_plot(
     else:
         raise ValueError('y has to be a 1d or 2d array')
 
+    #plt.show()
+    ticks_x_are_applied = False
+    if kwargs.get('labels_ticks_x') is None:
+        # max_x = data.shape[dict_axes['x']] - 1
+        if kwargs.get('ticks_x') is None:
+            if kwargs.get('n_ticks_x') is None:
+                pass
+            else:
+                max_x = x.max()
+                min_x = x.min()
+                n_labels_ticks_x = kwargs['n_ticks_x']
+                delta_x = (max_x - min_x) / (n_labels_ticks_x - 1)
+                tick_x_lower = min_x
+                tick_x_higher = max_x
+                kwargs['ticks_x'] = np.arange(tick_x_lower, tick_x_higher + delta_x, delta_x)
+                ax.set_xticks(kwargs['ticks_x'])
+                ticks_x_are_applied = True
+        else:
+            ax.set_xticks(kwargs['ticks_x'])
+            ticks_x_are_applied = True
+
+        if (kwargs.get('stagger_labels_ticks_x') or
+                (kwargs.get('fontsize_labels_ticks_x') is not None) or
+                (kwargs.get('rotation_labels_ticks_x') is not None)):
+            fig.canvas.draw()
+            tmp_labels_ticks_x = ax.get_xticklabels()[1:-1:1]
+            n_labels_ticks_x = len(tmp_labels_ticks_x)
+            kwargs['labels_ticks_x'] = [None] * n_labels_ticks_x
+            for l, label_l in enumerate(tmp_labels_ticks_x):
+                kwargs['labels_ticks_x'][l] = label_l.get_text()
+
+    if kwargs.get('labels_ticks_x') is not None:
+        if not ticks_x_are_applied:
+            if kwargs.get('ticks_x') is None:
+                max_x = x.max()
+                min_x = x.min()
+                n_labels_ticks_x = len(kwargs['labels_ticks_x'])
+                delta_x = (max_x - min_x) / (n_labels_ticks_x - 1)
+                tick_x_lower = min_x
+                tick_x_higher = max_x
+                kwargs['ticks_x'] = np.arange(tick_x_lower, tick_x_higher + delta_x, delta_x)
+
+            ax.set_xticks(kwargs['ticks_x'])
+        if kwargs.get('stagger_labels_ticks_x'):
+            kwargs['labels_ticks_x'] = (
+                [str(l) if not i % 2 else '\n' + str(l) for i, l in enumerate(kwargs['labels_ticks_x'])])
+        ax.set_xticklabels(
+            kwargs['labels_ticks_x'], fontsize=kwargs.get('fontsize_labels_ticks_x'),
+            rotation=kwargs.get('rotation_labels_ticks_x'))
+
+    ticks_y_are_applied = False
+    if kwargs.get('labels_ticks_y') is None:
+        # max_y = data.shape[dict_axes['y']] - 1
+        if kwargs.get('ticks_y') is None:
+            if kwargs.get('n_ticks_y') is None:
+                pass
+            else:
+                max_y = y.max()
+                min_y = y.min()
+                n_labels_ticks_y = kwargs['n_ticks_y']
+                delta_y = (max_y - min_y) / (n_labels_ticks_y - 1)
+                tick_y_lower = min_y
+                tick_y_higher = max_y
+                kwargs['ticks_y'] = np.arange(tick_y_lower, tick_y_higher + delta_y, delta_y)
+                ax.set_yticks(kwargs['ticks_y'])
+                ticks_y_are_applied = True
+
+        else:
+            ax.set_yticks(kwargs['ticks_y'])
+            ticks_y_are_applied = True
+
+        if (kwargs.get('stagger_labels_ticks_y') or
+                (kwargs.get('fontsize_labels_ticks_y') is not None) or
+                (kwargs.get('rotation_labels_ticks_y') is not None)):
+            fig.canvas.draw()
+            tmp_labels_ticks_y = ax.get_yticklabels()[1:-1:1]
+            n_labels_ticks_y = len(tmp_labels_ticks_y)
+            kwargs['labels_ticks_y'] = [None] * n_labels_ticks_y
+            for l, label_l in enumerate(tmp_labels_ticks_y):
+                kwargs['labels_ticks_y'][l] = label_l.get_text()
+
+    if kwargs.get('labels_ticks_y') is not None:
+        if not ticks_y_are_applied:
+            if kwargs.get('ticks_y') is None:
+                max_y = y.max()
+                min_y = y.min()
+                n_labels_ticks_y = len(kwargs['labels_ticks_y'])
+                delta_y = (max_y - min_y) / (n_labels_ticks_y - 1)
+                tick_y_lower = min_y
+                tick_y_higher = max_y
+                kwargs['ticks_y'] = np.arange(tick_y_lower, tick_y_higher + delta_y, delta_y)
+
+            ax.set_yticks(kwargs['ticks_y'])
+        if kwargs.get('stagger_labels_ticks_y'):
+            kwargs['labels_ticks_y'] = (
+                [l if not i % 2 else '\n' + l for i, l in enumerate(kwargs['labels_ticks_y'])])
+        ax.set_yticklabels(
+            kwargs['labels_ticks_y'], fontsize=kwargs.get('fontsize_labels_ticks_y'),
+            rotation=kwargs.get('rotation_labels_ticks_y'))
+
     if legend:
         ax.legend()
 
@@ -587,8 +597,8 @@ def points_1_format_in_1_plot(
             if kwargs.get('n_ticks_x') is None:
                 pass
             else:
-                max_x = x.max
-                min_x = x.min
+                max_x = x.max()
+                min_x = x.min()
                 n_labels_ticks_x = kwargs['n_ticks_x']
                 delta_x = (max_x - min_x) / (n_labels_ticks_x - 1)
                 tick_x_lower = min_x
@@ -604,7 +614,7 @@ def points_1_format_in_1_plot(
                 (kwargs.get('fontsize_labels_ticks_x') is not None) or
                 (kwargs.get('rotation_labels_ticks_x') is not None)):
             fig.canvas.draw()
-            tmp_labels_ticks_x = ax.get_xticklabels()
+            tmp_labels_ticks_x = ax.get_xticklabels()[1:-1:1]
             n_labels_ticks_x = len(tmp_labels_ticks_x)
             kwargs['labels_ticks_x'] = [None] * n_labels_ticks_x
             for l, label_l in enumerate(tmp_labels_ticks_x):
@@ -613,8 +623,8 @@ def points_1_format_in_1_plot(
     if kwargs.get('labels_ticks_x') is not None:
         if not ticks_x_are_applied:
             if kwargs.get('ticks_x') is None:
-                max_x = x.max
-                min_x = x.min
+                max_x = x.max()
+                min_x = x.min()
                 n_labels_ticks_x = len(kwargs['labels_ticks_x'])
                 delta_x = (max_x - min_x) / (n_labels_ticks_x - 1)
                 tick_x_lower = min_x
@@ -636,8 +646,8 @@ def points_1_format_in_1_plot(
             if kwargs.get('n_ticks_y') is None:
                 pass
             else:
-                max_y = y.max
-                min_y = y.min
+                max_y = y.max()
+                min_y = y.min()
                 n_labels_ticks_y = kwargs['n_ticks_y']
                 delta_y = (max_y - min_y) / (n_labels_ticks_y - 1)
                 tick_y_lower = min_y
@@ -654,7 +664,7 @@ def points_1_format_in_1_plot(
                 (kwargs.get('fontsize_labels_ticks_y') is not None) or
                 (kwargs.get('rotation_labels_ticks_y') is not None)):
             fig.canvas.draw()
-            tmp_labels_ticks_y = ax.get_yticklabels()
+            tmp_labels_ticks_y = ax.get_yticklabels()[1:-1:1]
             n_labels_ticks_y = len(tmp_labels_ticks_y)
             kwargs['labels_ticks_y'] = [None] * n_labels_ticks_y
             for l, label_l in enumerate(tmp_labels_ticks_y):
@@ -663,8 +673,8 @@ def points_1_format_in_1_plot(
     if kwargs.get('labels_ticks_y') is not None:
         if not ticks_y_are_applied:
             if kwargs.get('ticks_y') is None:
-                max_y = y.max
-                min_y = y.min
+                max_y = y.max()
+                min_y = y.min()
                 n_labels_ticks_y = len(kwargs['labels_ticks_y'])
                 delta_y = (max_y - min_y) / (n_labels_ticks_y - 1)
                 tick_y_lower = min_y
@@ -752,9 +762,15 @@ def heatmap(
             dict_axes, keys_axes_expected, values_axes_expected, name_dictionary='dict_axes')
         cc_check_args.values_are_not_repeated(dict_axes, name_dictionary='dict_axes')
 
+        if dict_axes['x'] < dict_axes['y']:
+            data = data.T
+            dict_axes = dict(zip(keys_axes_expected, values_axes_expected))
+            shape_data = np.asarray(data.shape, dtype=int)
+
     if ax is None:
         figures_existing = plt.get_fignums()
         n_figures_new = 1
+        id_figures = None
         i = 0
         f = 0
         while f < n_figures_new:
@@ -880,7 +896,7 @@ def heatmap(
                 (kwargs.get('fontsize_labels_ticks_x') is not None) or
                 (kwargs.get('rotation_labels_ticks_x') is not None)):
             fig.canvas.draw()
-            tmp_labels_ticks_x = ax.get_xticklabels()
+            tmp_labels_ticks_x = ax.get_xticklabels()[1:-1:1]
             n_labels_ticks_x = len(tmp_labels_ticks_x)
             kwargs['labels_ticks_x'] = [None] * n_labels_ticks_x
             for l, label_l in enumerate(tmp_labels_ticks_x):
@@ -930,7 +946,7 @@ def heatmap(
                 (kwargs.get('fontsize_labels_ticks_y') is not None) or
                 (kwargs.get('rotation_labels_ticks_y') is not None)):
             fig.canvas.draw()
-            tmp_labels_ticks_y = ax.get_yticklabels()
+            tmp_labels_ticks_y = ax.get_yticklabels()[1:-1:1]
             n_labels_ticks_y = len(tmp_labels_ticks_y)
             kwargs['labels_ticks_y'] = [None] * n_labels_ticks_y
             for l, label_l in enumerate(tmp_labels_ticks_y):
@@ -1036,7 +1052,7 @@ def heatmap(
 
 
 def heatmaps_in_1_figure(
-        data, dict_axes=None, id_figure=None, ratio_fig=None, sharex=False, sharey=False,
+        data, dict_axes=None, id_figure=None, ratio_fig=None, sharex='none', sharey='none',
         array_annot=None, add_letters_to_titles=True,
         hspace=None, wspace=None, maximum_n_pixels_per_plot=500, **kwargs):
 
@@ -1236,7 +1252,7 @@ def heatmaps_in_1_figure(
 
 
 def heatmaps_in_multi_figures(
-        data, dict_axes=None, id_figures=None, ratio_fig=None, sharex=False, sharey=False,
+        data, dict_axes=None, id_figures=None, ratio_fig=None, sharex='none', sharey='none',
         array_annot=None, hspace=None, wspace=None, add_letters_to_titles=True,
         maximum_n_pixels_per_plot=500, **kwargs):
 
@@ -1317,7 +1333,7 @@ def heatmaps_in_multi_figures(
     dict_arguments = cc_format_args.format_shape_arguments(
         dict(hspace=hspace, add_letters_to_titles=add_letters_to_titles,
              maximum_n_pixels_per_plot=maximum_n_pixels_per_plot,
-             sharex=sharex, sharey=sharey, ratio_fig=ratio_fig),
+             sharex=sharex, sharey=sharey),
         [n_figures])
 
     hspace = dict_arguments['hspace']
@@ -1325,7 +1341,6 @@ def heatmaps_in_multi_figures(
     maximum_n_pixels_per_plot = dict_arguments['maximum_n_pixels_per_plot']
     sharex = dict_arguments['sharex']
     sharey = dict_arguments['sharey']
-    ratio_fig = dict_arguments['ratio_fig']
 
     n_kwargs = len(kwargs)
     if n_kwargs > 0:
@@ -1353,6 +1368,7 @@ def heatmaps_in_multi_figures(
             heatmaps_in_1_figure(
                 data[tuple(index)], dict_axes=dict_axes_next,
                 id_figure=id_figures[f], ratio_fig=ratio_fig, sharex=sharex[f], sharey=sharey[f],
+                array_annot=None,
                 hspace=hspace[f], maximum_n_pixels_per_plot=maximum_n_pixels_per_plot[f],
                 add_letters_to_titles=add_letters_to_titles[f],
                 ** dict(zip(keys, [value[tuple(index_kwargs)] for value in kwargs.values()])))
@@ -1367,7 +1383,7 @@ def heatmaps_in_multi_figures(
 
             heatmaps_in_1_figure(
                 data[tuple(index)], dict_axes=dict_axes_next,
-                id_figure=id_figures[f], sharex=sharex[f], sharey=sharey[f],
+                id_figure=id_figures[f], ratio_fig=ratio_fig, sharex=sharex[f], sharey=sharey[f],
                 array_annot=array_annot[tuple(index)],
                 hspace=hspace[f], maximum_n_pixels_per_plot=maximum_n_pixels_per_plot[f],
                 add_letters_to_titles=add_letters_to_titles[f],
