@@ -1,6 +1,6 @@
 import numpy as np
 import sys
-from ..maths import convert_to_int_or_float, convert_to_int
+from ..maths import convert_to_int_or_float, rint
 from ..array import Range
 from ..combinations import (
     conditions_to_combinations_on_the_fly_with_indexes as
@@ -173,10 +173,10 @@ class Rectangle:
 
         topleft_pg = Point(self.topleft).convert_to_pygame_coordinates(size_frame).point
         left_pg, top_pg = topleft_pg
-        left_pg = convert_to_int(left_pg)
-        top_pg = convert_to_int(top_pg)
-        width = convert_to_int(self.width)
-        height = convert_to_int(self.height)
+        left_pg = rint(left_pg)
+        top_pg = rint(top_pg)
+        width = rint(self.width)
+        height = rint(self.height)
         rect_pg = pygame.Rect(left_pg, top_pg, width, height)
         return rect_pg
 
@@ -334,7 +334,7 @@ class Frame:
         # alignment_x_low = alignment_x.lower()
         # if alignment_x_low in ['c', 'center', 'centre']:
         #     rect_destination_pg.centerx = point_pg.x
-        #     # rect_destination_pg.centerx = convert_to_int(point_pg.x + .5)
+        #     # rect_destination_pg.centerx = rint(point_pg.x + .5)
         # elif alignment_x_low in ['l', 'left']:
         #     rect_destination_pg.left = point_pg.x
         # elif alignment_x_low in ['r', 'right']:
@@ -345,7 +345,7 @@ class Frame:
         # alignment_y_low = alignment_y.lower()
         # if alignment_y_low in ['c', 'center', 'centre']:
         #     rect_destination_pg.centery = point_pg.y
-        #     # rect_destination_pg.centery = convert_to_int(point_pg.y + .5)
+        #     # rect_destination_pg.centery = rint(point_pg.y + .5)
         # elif alignment_y_low in ['b', 'bottom']:
         #     rect_destination_pg.bottom = point_pg.y
         # elif alignment_y_low in ['t', 'top']:
